import {
  Metadata, Loader, IDictionary, IEventParam,
} from '@zeedhi/core';

import {
  GridEditable, Card, Form, DialogService, Button, Select,
} from '@zeedhi/common';

export class Ger09800PesoProdutoUnidadeController {
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private apresentacaoIsVisible = false;

  public openProduct() {
    const grid: GridEditable = Metadata.getInstance('gridPesoProduto');
    const unidade = Metadata.getInstance<Select>('UNIDADEFORM').value;
    if (unidade !== null) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: unidade,
        },
      ];
      
      const buttonCancel = Metadata.getInstance<Button>('cancelButton');
      buttonCancel.disabled = true;
      
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else grid.datasource.data = [];
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFUnidadeDefault({ component }: IEventParam<Select>) {
    const values = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA'),
      NMFILIAL: localStorage.getItem('NMFILIALLOGADA'),
    };
    component.datasource.data = [values];
    component.setValue(localStorage.getItem('CDFILIALLOGADA'));
  }

  private cardApresentacaoIsVisible() {
    const card = Metadata.getInstance<Card>('cardApresentacao');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.apresentacaoIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.apresentacaoIsVisible = false;
    }
  }
  
  public salvaApresentacao() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const form = Metadata.getInstance<Form>('formFilter').value;
    const formApresentacao = Metadata.getInstance<Form>('formApresentacao');
    const editedRows: IDictionary = gridApresentacao.getEditedRows();
    editedRows[0].CDPRODUTO = form.CDPRODUTO.CDPRODUTO;
    
    if (formApresentacao.validate()) {
      this.crudController.saveRows(editedRows, 'apreprod', gridApresentacao).then(() => {
        this.gridEditController.defaultState(gridApresentacao);
        gridApresentacao.datasource.get();
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public async deleteCheckedRows() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const checkedRows = gridApresentacao.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'confirmDeleteApresentacao',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteApresentacao() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteApresentacao() {
    const gridApresentacao: GridEditable = Metadata.getInstance('gridApresentacao');
    const checkedRows = gridApresentacao.selectedRows;
    const dataSourceName = '/apreprod';
    const buttonYes = Metadata.getInstance<Button>('yesDelete');
    this.utilities.activeLoadingButton(buttonYes);
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridApresentacao.cancelEditedRows();
      gridApresentacao.datasource.get();
      gridApresentacao.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    }).finally(() => {
      this.utilities.disableLoadingButton(buttonYes);
    });
  }
  
  public reloadApresentacao() {
    const formFiltro: Form = Metadata.getInstance<Form>('formFilter');
    if (formFiltro.validate()) {
      const gridApresentacao = Metadata.getInstance<GridEditable>('gridApresentacao');
      gridApresentacao.datasource.get();
    }
  }
  
  public closeCardProduto() {
    const card = Metadata.getInstance<Card>('cardApresentacao');
    
    card.isVisible = false;
    this.apresentacaoIsVisible = false;
  }
  
  public apresentacaoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDAPRERECE',
      },
      'NMAPRERECE',
    ],
    formatterDataText: [
      {
        name: 'CDAPRERECE',
      },
      'NMAPRERECE',
    ],
    dataValueOut: [
      {
        column: 'CDAPRERECE',
        columnOnGrid: 'CDAPRERECE',
      },
      {
        column: 'NMAPRERECE',
        columnOnGrid: 'NMAPRERECE',
      },
    ],
    dataValue: 'CDAPRERECE',
    dense: true,
    datasource: {
      uniqueKey: 'CDAPRERECE',
      route: '/aprerece',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
