import {
  Datasource, Http, IDictionary, Loader, Metadata, IEventParam, Config, I18n, dayjs as moment,
} from '@zeedhi/core';
import {
  Button, Date as ZdDate, Form, GridEditable, Select, TextInput, CheckboxMultiple,
  DialogService, Grid, Modal, ModalService, IButton, Tabs,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme02000RetiradaInventarioController {
  private modalAutoRemoval!: Modal;

  private modalProdIrreg!: Modal;
  
  private modalProdIrregRetAuto!: Modal;
  
  private modalProdNaoPlanejado!: Modal;

  private loadingAutoRemoval = false;

  private nrlistaProd = '';

  private titleProdIrreg = '';

  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public gridEditController = Loader.getInstance('GridEditableController');

  // public paramFilial: IDictionary = {};

  public paramsAutoRemoval: IDictionary = {};
  
  private get titleProdIrregRetAuto() {
    return `${I18n.translate('pme::prodIrregulares')}`;
  }

  public async createModal() {
    this.modalController.createModal('pme/medicao/modals/formRomaneio');
    this.modalAutoRemoval = await ModalService.createFromJson('pme/medicao/modals/retiradaAutomatica', true);
    this.modalProdIrreg = await ModalService.createFromJson('pme/medicao/modals/gridProdIrreg', true);
    this.modalProdIrregRetAuto = await ModalService.createFromJson('pme/medicao/modals/gridProdIrregRetAuto', true);
    this.modalProdNaoPlanejado = await ModalService.createFromJson('pme/medicao/modals/gridProdNaoPlanejado', true);
  }

  public destroyModal() {
    this.modalController.destroyModal();
    this.modalAutoRemoval.destroy();
    this.modalProdIrreg.destroy();
    this.modalProdIrregRetAuto.destroy();
    this.modalProdNaoPlanejado.destroy();
  }

  public openModalAutoRemoval() {
    this.modalAutoRemoval.show();
  }

  public closeModalAutoRemoval() {
    if (!this.loadingAutoRemoval) this.modalAutoRemoval.hide();
  }

  public openModalProdIrreg() {
    this.modalProdIrreg.show();
  }
  
  public openModalProdIrregRetAuto() {
    this.modalProdIrregRetAuto.show();
  }
  
  public openModalProdNaoPlanejado() {
    this.modalProdNaoPlanejado.show();
  }
  
  public closeModalProdNaoPlanejado() {
    this.modalProdNaoPlanejado.hide();
  }

  public closeModalProdIrreg() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    this.modalProdIrreg.hide();
    this.closeModalAutoRemoval();
    gridRomaneio.datasource.get();
  }
  
  public closeModalProdIrregRetAuto() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    this.modalProdIrregRetAuto.hide();
    this.closeModalAutoRemoval();
    gridRomaneio.datasource.get();
  }

  public dtIni = '';

  public dtFin = '';

  private situation = {
    PENDENTE: 'S',
    REALIZADO: 'N',
  };

  private currentRow: IDictionary = {};
  
  public disableFields = false;
  
  public disableGridAdd = false;

  public showCost = 'N';
  
  public showItems = false;
  
  public itemSaved = false;

  public colsRomaneio = 12;

  public selectedItems: IDictionary = [];

  public prodEditable({ row }: IDictionary) {
    return row.__is_new;
  }

  public get visibilityItems() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    // if (gridRomaneio.datasource.currentRow.DTINVENTRETI) this.colsRomaneio = 12;
    // else this.colsRomaneio = 12;
    return !!gridRomaneio.datasource.currentRow.DTINVENTRETI;
  }
  
  public getConfigRetirada() {
    this.disableFields = true;
    const central = this.appController.getCdCentral();
    if (central) {
      Http.post('/configRetirada', { CDCENTRAL: central }).then((response) => {
        const config = response.data.dataset.result;
        if (config[0] && config[0].IDUTRETBASEINVE === 'S') {
          this.disableFields = false;
          this.disableGridAdd = false;
          this.utilities.getProcessDate(central).then((date: string) => {
            this.dtIni = date;
            this.dtFin = date;
          });
        } else {
          this.disableFields = true;
          this.disableGridAdd = true;
          this.utilities.showAlert('erroConfig', 'pme::centralNaoHabilitadaRetirada', 'error');
        }
      });
    }
  }

  public setSituation({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;

      if (items.includes('PENDENTE')) this.situation.PENDENTE = 'S';
      else this.situation.PENDENTE = 'N';
      
      if (items.includes('REALIZADO')) this.situation.REALIZADO = 'S';
      else this.situation.REALIZADO = 'N';

      this.setFilterRomaneio(component);
    }
  }

  public setCost({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;
      this.showCost = items.includes('SHOWCUSTO') ? 'S' : 'N';
      this.reloadItems();
    }
  }
  
  public validateAddRomaneio() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    Http.post('/configRetirada', { CDCENTRAL: agregada.value }).then((response) => {
      if (response.data.dataset.result[0].IDPRINVENRETI === 'S' && this.disableFields === false) this.disableGridAdd = false;
      else this.disableGridAdd = true;
    });
  }

  public setFilterRomaneio({ component }: IDictionary) {
    if (component.name === 'UNIDADE') this.validateAddRomaneio();
    const grid: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    
    // this.findParamFilial();

    let efetivados = null;
    if (this.situation.PENDENTE === 'S' && this.situation.REALIZADO === 'S') efetivados = 'T';
    else if (this.situation.PENDENTE === 'S') efetivados = 'N';
    else if (this.situation.REALIZADO === 'S') efetivados = 'S';

    const conditions = [
      {
        name: 'P_CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'P_DTINICIAL',
        operator: '=',
        value: this.dtIni,
      },
      {
        name: 'P_DTFINAL',
        operator: '=',
        value: this.dtFin,
      },
      {
        name: 'P_IDRETIEFET',
        operator: '=',
        value: efetivados,
      },
    ];

    grid.datasource.filter = conditions;
    grid.datasource.get();
  }

  public setFilterItems({ component }: IEventParam<Datasource>) {
    this.currentRow = component.currentRow;

    const addReti = Metadata.getInstance<Button>('addButtonReti');
    const addItem = Metadata.getInstance<Button>('addButtonItem');
    
    addReti.disabled = this.disableGridAdd;
    addItem.disabled = this.disableGridAdd;
    
    this.reloadItems();
  }

  public reloadItems() {
    const grid: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const central = this.appController.getCdCentral();
    // const dtInvent = moment(this.currentRow.DTINVENTRETI, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    const dtInvent = moment(this.currentRow.DTINVENTRETI, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    // if (this.paramFilial && (this.paramFilial.IDORDAJUINVE === null ||
    // this.paramFilial.IDORDAJUINVE === 'P')) date =
    // moment(dtInvent, 'DD/MM/YYYY').add(1, 'd').format('DD/MM/YYYY');
    // else date = dtInvent;
    
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: agregada.value,
        },
        {
          name: 'P_IDEXIBCUS',
          operator: '=',
          value: this.showCost,
        },
        {
          name: 'P_DTPOSIESTQ',
          operator: '=',
          value: dtInvent,
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: central,
        },
        {
          name: 'P_NRINVENTRETI',
          operator: '=',
          value: this.currentRow.NRINVENTRETI,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get().then(() => {
        document.getElementsByClassName('gridRomaneioItems')[0].scrollIntoView();
      });
    }
  }

  public getFilterProdItem() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.currentRow.NRINVENTRETI ? this.currentRow.NRINVENTRETI : null,
      },
    ];
  }
  
  public setFilterService() {
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.paramsAutoRemoval.CDFILIAL ? this.paramsAutoRemoval.CDFILIAL : null,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilities.isMerenda() ? '' : 'N',
      },
      {
        name: 'IDSERVATIV',
        operator: '=',
        value: 'S',
      },
    ];
    return conditions;
  }

  public setFilterProdForm({ component }: IEventParam<Select>) {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: agregada.value,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.romaneio.NRINVENTRETI ? this.romaneio.NRINVENTRETI : null,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public saveItems({ component }: IDictionary) {
    const formFiltro = Metadata.getInstance<Form>('formFiltro');
    const formItemsGrid = Metadata.getInstance<Form>('formItemsGrid');
    if (this.currentRow.IDRETIEFET === 'S') this.utilities.showAlert('errorEdicaoItem', 'pme::romaneioJaEfetivado', 'error');
    else if (formFiltro.validate() && formItemsGrid.validate()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      const rows: IDictionary = grid.getEditedRows();
      if (rows.length > 0) {
        rows.forEach((e: IDictionary) => {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDFILIAL = formFiltro.value.UNIDADE;
          e.DTINVENTRETI = this.currentRow.DTINVENTRETI;
          e.NRINVENTRETI = this.currentRow.NRINVENTRETI;
        });
        this.callSave('/saveItemsRomaneio', rows, grid);
      }
    }
  }

  public deleteItems({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    this.selectedItems = grid.selectedRows;
    if (this.currentRow.IDRETIEFET === 'N') {
      if (this.selectedItems.length === grid.datasource.total) {
        this.selectedItems = grid.selectedRows;
        DialogService.show({
          name: `${component.name}Lancamento`,
          type: 'warning',
          title: 'pme::atencao',
          text: 'pme::excluirUltimoItemLancamento',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'pme::sim',
              events: { click: () => this.callDeleteItems() },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'pme::nao',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      } else this.gridEditController.deleteCheckedRows({ component });
    } else this.utilities.showAlert('errorExclusaoItem', 'pme::romaneioJaEfetivado', 'error');
  }

  public callDeleteItems() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const agregada: Select = Metadata.getInstance('UNIDADE');
    this.selectedItems[0].CDFILIAL = agregada.value;

    Http.post('/deleteItemRomaneio', { requestType: 'Row', row: this.selectedItems }).then(() => {
      gridItem.cancelEditedRows();
      gridItem.datasource.get();
      gridItem.selectedRows = [];
      this.selectedItems = [];
      gridRomaneio.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public deleteRomaneio({ component }: IEventParam<Button>) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const checkedRows = gridRomaneio.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else if (!this.romaneioIsValid(checkedRows)) this.utilities.showAlert('alert', 'pme::romaneioJaEfetivado', 'warning');
    else {
      DialogService.show({
        name: `${component.name}`,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::excluirLancamento',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteRomaneio() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  public romaneioIsValid(rows: IDictionary): boolean {
    let valid = true;
    rows.forEach((e: IDictionary) => {
      if (e.IDRETIEFET === 'S') valid = false;
    });
    return valid;
  }

  public callDeleteRomaneio() {
    const gridItem: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const rows = gridRomaneio.selectedRows;

    Http.post('/deleteRomaneio', { requestType: 'Row', row: rows }).then(() => {
      gridRomaneio.cancelEditedRows();
      gridRomaneio.datasource.get();
      gridRomaneio.selectedRows = [];
      this.selectedItems = [];
      gridItem.datasource.get();
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  public itemIsValid({ row }: IDictionary) {
    let color = '';
    if (this.currentRow.IDRETIEFET === 'N' && row.QTESTOQDIA < row.QTITINVENTRETI) color = 'red';
    return color;
  }

  public cssEfetivado({ row }: IDictionary) {
    let colorClass = 'green-efetivos';
    if (row.IDRETIEFET === 'N') colorClass = 'red-efetivos';
    return colorClass;
  }

  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'SGUNIDADE',
        columnOnGrid: 'SGUNIDADE',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodUltimoNivelItem',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    if (grid) grid.datasource.loading = true;
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.utilities.showAlert('alertSuccess', 'pme::prodIncluidoNaRetirada', 'green');
        this.gridEditController.defaultState(grid);
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      if (grid) grid.datasource.loading = false;
    });
  }

  get maskProd() {
    return [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ];
  }

  // modalRomaneio functions
  private romaneio: IDictionary = {};

  public showModalItems = false;

  public get notShowModalItems() {
    return !this.showModalItems;
  }

  public addRomaneio() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    if (agregada.value) {
      this.showModalItems = false;
      this.modalController.openModal();
      // this.focusOnData();
    } else this.utilities.showAlert('errorUnidade', 'pme::unidadeNaoSelecionada', 'error');
  }

  public closeRomaneio(closeDialog = false) {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    
    this.romaneio = {};
    this.itemSaved = false;
    if (closeDialog) DialogService.hide();
    this.modalController.closeModal();
    gridRomaneio.datasource.get();
    gridItems.datasource.get();
  }

  public focusOnData({ component }: IEventParam<ZdDate>) {
    component.value = this.utilities.dtProcessamento;
    setTimeout(() => {
      this.utilities.focusOnField('DATEROMANEIO');
    }, 1);
  }

  public confirmaRomaneio() {
    const formRomaneio = Metadata.getInstance<Form>('formRomaneio');
    if (formRomaneio.validate()) {
      const agregada: Select = Metadata.getInstance('UNIDADE');
      this.romaneio = {
        CDFILIAL: agregada.value,
        DTINVENTRETI: formRomaneio.value.DATEROMANEIO,
        DSINVENTRETI: formRomaneio.value.DESCRIPTION,
        IDRETIEFET: 'N',
        NRINVENTRETI: null,
      };
      this.showModalItems = true;
      setTimeout(() => {
        this.utilities.focusOnField('PRODUCT');
      }, 1);
    }
  }

  public saveItemForm() {
    const formItems = Metadata.getInstance<Form>('formItems');
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (formItems.validate()) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: formItems.value.PRODUCT,
          QTITINVENTRETI: formItems.value.QTD,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      
      Http.post('/saveRomaneio', row).then((response) => {
        this.itemSaved = true;
        this.romaneio.NRINVENTRETI = response.data.dataset.nrseqinvent[0];
        this.utilities.showAlert('alert', 'pme::prodIncluidoNaRetirada', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('error', error, 'error');
      });

      product.setValue('');
      qtd.value = '';
      this.utilities.focusOnField('PRODUCT');
    }
  }

  public confirmFormRomaneio({ component }: IEventParam<Button>) {
    const product = Metadata.getInstance<Select>('PRODUCT');
    const qtd = Metadata.getInstance<TextInput>('QTD');
    
    if (product.value && qtd.value) {
      const row = {
        romaneio: this.romaneio,
        item: {
          CDFILIAL: this.romaneio.CDFILIAL,
          CDPRODUTO: product.value,
          QTITINVENTRETI: qtd.value,
          NRINVENTRETI: this.romaneio.NRINVENTRETI,
        },
      };
      this.utilities.activeLoadingButton(component);
      Http.post('/saveRomaneio', row).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.closeRomaneio(false);
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    } else if (!this.itemSaved) {
      DialogService.show({
        name: 'finalizarRomaneio',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::cancelarInclusaoRomaneio',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.closeRomaneio(true) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else this.closeRomaneio(false);
  }

  /**
   * Rotina de Retirada Automática
   */
  private activatedButton: IButton | undefined;

  public get disableAutoRemoval() {
    return this.currentRow.IDRETIEFET !== 'N';
  }

  // gravar
  public recordStock({ component }: IEventParam<Button>) {
    this.activatedButton = component;
    this.currentRow.ZERAR = false;
    // if (this.stockIsValid()) this.validateAutoRemoval(this.currentRow);
    this.validateAutoRemoval(this.currentRow);
  }

  // zerar
  public resetStock({ component }: IEventParam<Button>) {
    this.activatedButton = component;
    this.currentRow.ZERAR = true;
    // if (this.stockIsValid()) this.validateAutoRemoval(this.currentRow);
    this.validateAutoRemoval(this.currentRow);
  }

  private validateAutoRemoval(row: IDictionary) {
    this.utilities.activeLoadingButton(this.activatedButton);
    this.loadingAutoRemoval = true;
    this.paramsAutoRemoval.DTVALESTOQ = this.paramsAutoRemoval.DTINVENTRETI;
    Http.post('/validateAutoRemoval', row).then((response) => {
      this.paramsAutoRemoval = response.data.dataset.params[0];
      const dtIni = moment(this.paramsAutoRemoval.DTINICIAL, 'DD/MM/YYYY');
      let dtInvent = moment(this.paramsAutoRemoval.DTINVENTRETI, 'DD/MM/YYYY');
      let msg = '';
      if (dtInvent.diff(dtIni, 'day') >= 60) {
        dtInvent = dtInvent.subtract(60, 'day');
        this.paramsAutoRemoval.DTINICIAL = dtInvent.format('DD/MM/YYYY');
      
        msg = 'pme::dtInventSup60Dias';
        this.dialogDtIni(msg, '60days');
      } else if (dtInvent.diff(dtIni, 'day') >= 30) {
        dtInvent = dtInvent.subtract(30, 'day');
        this.paramsAutoRemoval.DTINICIAL = dtInvent.format('DD/MM/YYYY');
      
        msg = 'pme::dtInventSup30Dias';
        this.dialogDtIni(msg, '30days');
      } else this.callAutoRemoval();
    }).catch((error: string) => {
      const arrError = error.split('_');
      const msgError = arrError[0];
      
      if (msgError === 'pme::prodNaoControlaEstoq') {
        this.titleProdIrreg = 'pme::prodNaoControlaEstoq';
        this.utilities.showAlert('error', 'pme::msgProdNaoControlaEstoq', 'error');
      } else if (msgError === 'pme::prodInativoUnidade') {
        this.titleProdIrreg = 'pme::produtosInativos';
        this.utilities.showAlert('error', 'pme::prodInativoUnidade', 'error');
      } else this.utilities.showAlert('error', msgError, 'error');
      
      if (msgError.includes('pme')) {
        this.nrlistaProd = arrError[1];
        this.openModalProdIrreg();
      }
    }).finally(() => {
      this.utilities.disableLoadingButton(this.activatedButton);
      this.loadingAutoRemoval = false;
    });
  }

  private dialogDtIni(text = '', name = 'dialog') {
    DialogService.show({
      name,
      type: 'warning',
      title: 'pme::dtUltInventario',
      text,
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'pme::sim',
          events: { click: () => this.callAutoRemoval() },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  private dialogEfet() {
    DialogService.show({
      name: `dialogEfet${this.paramsAutoRemoval.CDFILIAL + this.paramsAutoRemoval.NRINVENTRETI}`,
      type: 'warning',
      title: 'pme::efetivosPeriodo',
      text: 'pme::semEfetivosPeriodoInvent',
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'pme::sim',
          events: {
            click: () => {
              this.paramsAutoRemoval.ERROR = null;
              this.paramsAutoRemoval.IDGERARETI = 'N';
              this.callAutoRemoval();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'pme::nao',
          events: {
            click: () => {
              this.deleteConsplan(this.paramsAutoRemoval);
              DialogService.hide();
            },
          },
        },
      ],
    });
  }
  
  private dialogUnplannedProd() {
    DialogService.show({
      name: 'infoProdNaoPlan',
      type: 'info',
      title: 'pme::produtosNaoPlanejados',
      text: 'pme::prodNaoPlanejadoRetirada',
      persistent: false,
      // text: `${I18n.translate('pme::servRetiradaAutomaticaMensagem',
      // { PARAM1: servRet.CDSERVRETPME, PARAM2: servRet.NMSERVICO })}`,
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'pme::sim',
          events: {
            click: () => {
              this.paramsAutoRemoval.ERROR = null;
              this.paramsAutoRemoval.checkUnplannedProd = false;
              this.callAutoRemoval();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'pme::informarServicos',
          events: {
            click: () => {
              this.openModalProdNaoPlanejado();
              DialogService.hide();
            },
          },
        },
        {
          name: 'cancelRet',
          component: 'ZdButton',
          label: 'pme::cancelar',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  private stockIsValid() {
    let valid = true;
    let text = '';
    const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
    if (gridItems.datasource.data[0].ESTOQSUFICIENTE === 'N') {
      valid = false;
      text = 'pme::qtdEstoqIndisponivel';
    } else if (gridItems.datasource.data[0].ESTOQEQUIVALENTE === 'S') {
      valid = false;
      text = 'pme::qtEstoqEquivalente';
    }

    if (!valid) {
      DialogService.show({
        name: 'estoqueInsuficiente',
        type: 'warning',
        title: 'pme::atencao',
        text,
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: 'pme::OK',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
    return valid;
  }

  public setFilterModalProdNEstoq() {
    const grid: Grid = Metadata.getInstance('gridProdNaoControlaEstoq');
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: this.currentRow.CDFILIAL,
        },
        {
          name: 'P_NRINVENTRETI',
          operator: '=',
          value: this.currentRow.NRINVENTRETI,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get();
    }
  }

  public setFilterModalProdIrreg() {
    const grid: Grid = Metadata.getInstance('gridProdIrreg');
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'NRLISTA',
          operator: '=',
          value: this.nrlistaProd,
        },
      ];
  
      grid.datasource.filter = conditions;
      grid.datasource.get();
    }
  }
  
  public setFilterModalProdNaoPlanejado() {
    const grid: Grid = Metadata.getInstance('gridProdNaoPlanejado');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.paramsAutoRemoval.CDFILIAL ? this.paramsAutoRemoval.CDFILIAL : null,
      },
      {
        name: 'NRINVENTRETI',
        operator: '=',
        value: this.paramsAutoRemoval.NRINVENTRETI ? this.paramsAutoRemoval.NRINVENTRETI : null,
      },
      {
        name: 'NRCONSPLAN',
        operator: '=',
        value: this.paramsAutoRemoval.NRCONSPLAN ? this.paramsAutoRemoval.NRCONSPLAN : null,
      },
    ];

    grid.datasource.filter = conditions;
    grid.datasource.get();
  }
  
  public setFilterModalProdIrregRetAuto({ component }: IEventParam<Grid>) {
    let type = 'I';
    if (component.name === 'prodSemPosEstoq') type = 'E';
    
    if (this.currentRow.NRINVENTRETI) {
      const conditions = [
        {
          name: 'NRLISTA',
          operator: '=',
          value: this.nrlistaProd,
        },
        {
          name: 'TIPO',
          operator: '=',
          value: type,
        },
      ];
  
      component.datasource.filter = conditions;
      component.datasource.get().then((result: IDictionary) => {
        if (result.length === 0) {
          const tab = Metadata.getInstance<Tabs>('tabsProds');
          if (type === 'I') tab.activeTab = 1;
        }
      });
    }
  }

  private callAutoRemoval() {
    this.paramsAutoRemoval.CDFILCENTPME = this.appController.getCdCentral();
    this.paramsAutoRemoval.DTPROCESSA = this.utilities.dtProcessamento;
    if (this.paramsAutoRemoval.checkUnplannedProd !== false) this.paramsAutoRemoval.checkUnplannedProd = true;
    this.utilities.activeLoadingButton(this.activatedButton);
    this.loadingAutoRemoval = true;
    Http.post('/autoRemoval', this.paramsAutoRemoval).then((response) => {
      const result = response.data.dataset?.params[0];
      // const servRet = response.data.dataset?.params[1];
      const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
      if (result) {
        this.paramsAutoRemoval = result;
        if (result.ERROR && result.ERROR === 'pme::produtosNaoPlanejados') {
          this.dialogUnplannedProd();
        } else if (result.ERROR && result.ERROR === 'pme::semEfetivosPeriodoInvent') {
          this.dialogEfet();
        } else {
          this.nrlistaProd = result;
          this.openModalProdIrregRetAuto();
        }
      } else {
        gridRomaneio.datasource.get().then(() => {
          // const gridItems: GridEditable = Metadata.getInstance('romaneioItemsGrid');
          // const currentRomaneio = gridRomaneio.datasource.currentRow;
          // gridRomaneio.datasource.currentRow = currentRomaneio;
          // gridItems.datasource.get();
          this.closeModalAutoRemoval();
          this.utilities.showAlert('alert', 'pme::processoRealComSucesso', 'success');

          // if (servRet) {
            
          // }
        });
      }
    }).catch((error: string) => {
      // if (error.indexOf('pme::produtosNaoPlanejados') >= 0) this.dialogUnplannedProd();
      // else
      this.utilities.showAlert('error', error, 'error');
    }).finally(() => {
      this.utilities.disableLoadingButton(this.activatedButton);
      this.loadingAutoRemoval = false;
    });
  }

  private deleteConsplan(params: IDictionary) {
    Http.post('/deleteConsplanRetirada', params);
  }

  public closeItems() {
    const gridRomaneio: GridEditable = Metadata.getInstance('romaneioGrid');
    gridRomaneio.datasource.currentRow = {};
  }

  /*
  public findParamFilial() {
    const agregada: Select = Metadata.getInstance('UNIDADE');
    const row = {
      CDFILIAL: agregada.value,
    };
    
    Http.post('/findParamFilial', { requestType: 'Row', row }).then((response: IDictionary) => {
      const params = response.data.dataset.paramfilial[0];
      this.paramFilial = params;
    });
  }
  */
  
  public saveModalProdNaoPlanejado() {
    const grid: GridEditable = Metadata.getInstance('gridProdNaoPlanejado');
    const editedRows = grid.getEditedRows() as IDictionary;
    const rows = grid.datasource.data;
    rows.forEach((row: IDictionary) => {
      const editedRow = editedRows.find((el: IDictionary) => el.CDPRODUTO === row.CDPRODUTO);
      if (editedRow) row.CDSERVFILI = editedRow.CDSERVFILI;
    });
    this.paramsAutoRemoval.ERROR = null;
    this.paramsAutoRemoval.unplannedItems = rows;
    this.callAutoRemoval();
    this.closeModalProdNaoPlanejado();
  }
  
  public isDetailsActionVisible({ row }: IDictionary) {
    return row.IDORIGLANCPRINV === '1';
  }
}
